<?php

/**
 * The file of the EventVenueshowcase class.
 *
 * @link       https://themeatelier.net
 * @since      1.0.0
 *
 * @package event-venue-showcase
 */

namespace ThemeAtelier\EventVenueShowcase;

use ThemeAtelier\EventVenueShowcase\Loader;
use ThemeAtelier\EventVenueShowcase\Admin\Admin;
use ThemeAtelier\EventVenueShowcase\Frontend\Frontend;
use ThemeAtelier\EventVenueShowcase\Traits\Singleton;

// don't call the file directly.
if (!defined('ABSPATH')) {
	exit;
}

/**
 * The main class of the plugin.
 *
 * Handle all the class and methods of the plugin.
 *
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class EventVenueShowcase
{

	/**
	 * Plugin version
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var string
	 */
	protected $version;

	/**
	 * Plugin slug
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var string
	 */
	protected $plugin_slug;

	/**
	 * Main Loader.
	 *
	 * The loader that's responsible for maintaining and registering all hooks that empowers
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var object
	 */
	protected $loader;

	use Singleton;


	/**
	 * Constructor for the EventVenueshowcase class.
	 *
	 * Sets up all the appropriate hooks and actions within the plugin.
	 */
	public function __construct()
	{
		$this->version     = EVENT_VENUE_SHOWCASE_VERSION;
		$this->plugin_slug = 'event-venue-showcase';
		$this->load_dependencies();
		$this->define_constants();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		add_action('plugin_loaded', array($this, 'init_plugin'));
		add_action('admin_init', array($this, 'event_venue_plugin_check'));



		$active_plugins = get_option('active_plugins');
		foreach ($active_plugins as $active_plugin) {
			$_temp = strpos($active_plugin, 'event-venue-showcase.php');
			if (false != $_temp) {
				add_filter('plugin_action_links_' . $active_plugin, array($this, 'event_venue_plugin_action_links'));
			}
		}
	}

	/**
	 * Display notice if Event Venue Showcase plugin is not active.
	 */
	public function event_venue_required_notice()
	{
		?>
		<div class="notice notice-error">
			<p><b><?php echo esc_html__('The "Event Venue Showcase" is required "The Events Calendar plugin". Please install and activate "The Events Calendar".', 'event-venue-showcase'); ?></b></p>
		</div>
		<?php
	}

	/**
	 * Check if Event Venue Showcase plugin is active.
	 */
	public function event_venue_plugin_check()
	{
		// Check if the plugin is active
		if (!in_array('the-events-calendar/the-events-calendar.php', apply_filters('active_plugins', get_option('active_plugins')))) {
			if (!is_plugin_active('event-venue-showcase/plugin-main-file.php')) {
				add_action('admin_notices', array($this, 'event_venue_required_notice'));
			}
		}
	}

	public function event_venue_plugin_action_links($links)
	{
		$new_links = array(
			sprintf('<a href="%s">%s</a>', admin_url('edit.php?post_type=event_venue'), esc_html__('Settings', 'event-venue-showcase')),
		);
		return array_merge($new_links, $links);
	}

	/**
	 * Define the constants
	 *
	 * @return void
	 */
	public function define_constants()
	{
		define('EVENT_VENUE_SHOWCASE_URL', plugins_url('', EVENT_VENUE_SHOWCASE_FILE));
		define('EVENT_VENUE_SHOWCASE_FRONT', EVENT_VENUE_SHOWCASE_URL . '/includes/Frontend');
		define('EVENT_VENUE_SHOWCASE_ADMIN', EVENT_VENUE_SHOWCASE_URL . '/includes/Admin');
	}

	/**
	 * Load the plugin after all plugins are loaded.
	 *
	 * @return void
	 */
	public function init_plugin()
	{
		do_action('event_venue_loaded');
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Loader. Orchestrates the hooks of the plugin.
	 * - Teamproi18n. Defines internationalization functionality.
	 * - Admin. Defines all hooks for the admin area.
	 * - Frontend. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		$this->loader = new Loader();
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Frontend($this->get_plugin_slug(), $this->get_version());
		$this->loader->add_shortcode('event_venue', $plugin_public, 'render_shortcode');
		$this->loader->add_action('wp_loaded', $plugin_public, 'register_all_scripts');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
	}

	/**
	 * Register all of the hooks related to the admin dashboard functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{
		$plugin_admin = new Admin($this->get_plugin_slug(), $this->get_version());
		$this->loader->add_action('init', $plugin_admin, 'shortcode_post_type');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
		$this->loader->add_filter('manage_event_venue_posts_columns', $plugin_admin, 'set_venue_custom_column', 10, 2);
		$this->loader->add_action('manage_event_venue_posts_custom_column', $plugin_admin, 'get_venue_custom_column', 10, 2);
		$this->loader->add_filter('post_updated_messages', $plugin_admin, 'post_update_message');
		$this->loader->add_filter('term_updated_messages', $plugin_admin, 'update_venue_term_messages');
	}

	/**
	 * Initialize plugin for localization
	 *
	 * @uses load_plugin_textdomain()
	 */
	public function localization_setup()
	{
		load_plugin_textdomain('event-venue-showcase', false, dirname(plugin_basename(__FILE__)) . '/languages/');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The slug of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_slug()
	{
		return $this->plugin_slug;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
}
