<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @link       https://themeatelier.net
 * @since      1.0.0
 *
 * @package event-venue-showcase
 * @subpackage event-venue-showcase/Frontend
 * @author     ThemeAtelier<themeatelierbd@gmail.com>
 */

namespace ThemeAtelier\EventVenueShowcase\Frontend;

use ThemeAtelier\EventVenueShowcase\Frontend\Manager;

/**
 * The Frontend class to manage all public facing stuffs.
 *
 * @since 1.0.0
 */
class Frontend
{
	/**
	 * The slug of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_slug   The slug of this plugin.
	 */
	private $plugin_slug;

	/**
	 * The min of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $min   The slug of this plugin.
	 */
	private $min;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{
		add_shortcode('event_venue', array($this, 'render_shortcode'));

		$this->evs_public_action();
		$this->min         = defined('WP_DEBUG') && WP_DEBUG ? '' : '.min';
	}

	private function evs_public_action()
	{
		add_action('wp_ajax_evs_post_popup', array($this, 'evs_post_popup'));
		add_action('wp_ajax_nopriv_evs_post_popup', array($this, 'evs_post_popup'));

		add_action('wp_ajax_evs_post_admin_popup_next_prev', array($this, 'evs_post_admin_popup_next_prev'));
		add_action('wp_ajax_nopriv_evs_post_admin_popup_next_prev', array($this, 'evs_post_admin_popup_next_prev'));

		add_action('wp_ajax_load_more_events', [$this, 'load_more_events']);
		add_action('wp_ajax_nopriv_load_more_events', [$this, 'load_more_events']);
	}


	/**
	 * Load dynamic style of the existing shortcode id.
	 *
	 * @param  mixed $found_generator_id to push id option for getting how many shortcode in the page.
	 * @param  mixed $views_meta to push all options.
	 * @return array get dynamic style use in the specific shortcode.
	 */
	public static function load_dynamic_style($found_generator_id, $views_meta = '')
	{
		$custom_css = '';
		// If multiple shortcode found in the page.
		if (is_array($found_generator_id)) {
			foreach ($found_generator_id as $views_id) {
				if ($views_id && is_numeric($views_id) && get_post_status($views_id) !== 'trash') {
					$views_meta = get_post_meta($views_id, 'evs_venue_metaboxes', true);
					include 'partials/dynamic-style.php';
				}
			}
		} else {
			// If single shortcode found in the page.
			$views_id = $found_generator_id;
			include 'partials/dynamic-style.php';
		}
		$custom_style = trim(html_entity_decode(get_option('evs_venue_settings')['custom_css']));
		// Custom css merge with dynamic style.
		if (!empty($custom_style)) {
			$custom_css .= $custom_style;
		}
		$dynamic_style = array(
			'dynamic_css' => Manager::minify_output($custom_css),
		);
		return $dynamic_style;
	}

	/**
	 * Gets the existing shortcode-id, page-id and option-key from the current page.
	 *
	 * @return array
	 */
	public static function get_page_data()
	{
		$current_page_id    = get_queried_object_id();
		$option_key         = 'evs_venue_page_id' . $current_page_id;
		$found_generator_id = get_option($option_key);
		if (is_multisite()) {
			$option_key         = 'evs_venue_page_id' . get_current_blog_id() . $current_page_id;
			$found_generator_id = get_site_option($option_key);
		}
		$get_page_data = array(
			'page_id'      => $current_page_id,
			'generator_id' => $found_generator_id,
			'option_key'   => $option_key,
		);
		return $get_page_data;
	}

	/**
	 * If the option does not exist, it will be created.
	 *
	 * It will be serialized before it is inserted into the database.
	 *
	 * @param  string $post_id existing shortcode id.
	 * @param  array  $get_page_data get current page-id, shortcode-id and option-key from the page.
	 * @return void
	 */
	public static function db_options_update($post_id, $get_page_data)
	{
		$found_generator_id = $get_page_data['generator_id'];
		$option_key         = $get_page_data['option_key'];
		$current_page_id    = $get_page_data['page_id'];
		if ($found_generator_id) {
			$found_generator_id = is_array($found_generator_id) ? $found_generator_id : array($found_generator_id);
			if (!in_array($post_id, $found_generator_id) || empty($found_generator_id)) {
				// If not found the shortcode id in the page options.
				array_push($found_generator_id, $post_id);
				if (is_multisite()) {
					update_site_option($option_key, $found_generator_id);
				} else {
					update_option($option_key, $found_generator_id);
				}
			}
		} else {
			// If option not set in current page add option.
			if ($current_page_id) {
				if (is_multisite()) {
					add_site_option($option_key, array($post_id));
				} else {
					add_option($option_key, array($post_id));
				}
			}
		}
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in WBP_Category_Slider_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The WBP_Category_Slider_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$get_page_data      = self::get_page_data();
		$found_generator_id = $get_page_data['generator_id'];
		if ($found_generator_id) {
			wp_enqueue_style('evs_venue_swiper');
			wp_enqueue_style('magnific-popup');
			wp_enqueue_style('evs_venue_font_awesome');
			wp_enqueue_style('evs_venue_style');
			// Load dynamic style based on the existing shortcode ids in the current page.
			$dynamic_style = self::load_dynamic_style($found_generator_id);
			wp_add_inline_style('evs_venue_style', wp_strip_all_tags($dynamic_style['dynamic_css']));
		} else {
			wp_enqueue_style('evs_venue_style');
		}
	}

	/**
	 * Register the All scripts for the public-facing side of the site.
	 *
	 * @since    2.0
	 */
	public function register_all_scripts()
	{
		wp_register_style('magnific-popup', EVENT_VENUE_SHOWCASE_FRONT . '/assets/css/magnific-popup' . $this->min . '.css', array(), '1.1.0', 'all');
		wp_register_style('evs_venue_swiper', EVENT_VENUE_SHOWCASE_FRONT . '/assets/css/swiper-bundle' . $this->min . '.css', array(), '7.4.1', 'all');
		wp_register_style('evs_venue_font_awesome', EVENT_VENUE_SHOWCASE_FRONT . '/assets/css/font-awesome' . $this->min . '.css', array(), EVENT_VENUE_SHOWCASE_VERSION, 'all');
		wp_register_style('evs_venue_style', EVENT_VENUE_SHOWCASE_FRONT . '/assets/css/style' . $this->min . '.css', array(), EVENT_VENUE_SHOWCASE_VERSION, 'all');

		wp_register_script('swiper-bundle', EVENT_VENUE_SHOWCASE_FRONT . '/assets/js/swiper-bundle' . $this->min . '.js', array('jquery'), '7.4.1', true);
		wp_register_script('magnific-popup', EVENT_VENUE_SHOWCASE_FRONT . '/assets/js/jquery.magnific-popup' . $this->min . '.js', array('jquery'), '1.1.0', true);
		wp_register_script('evs-main', EVENT_VENUE_SHOWCASE_FRONT . '/assets/js/script' . $this->min . '.js', array('jquery', 'swiper-bundle', 'magnific-popup'), EVENT_VENUE_SHOWCASE_VERSION, true);
		wp_localize_script(
			'evs-main',
			'evs',
			array(
				'ajaxurl' => admin_url('admin-ajax.php'),
				'nonce'   => wp_create_nonce('evs_nonce'),
			)
		);
	}

	/**
	 * Post modal.
	 */
	public static function evs_post_popup()
	{
		if (isset($_POST['nonce']) && !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'])), 'evs_nonce')) {
			return false;
		}

		$post_id = isset($_POST['post_id']) ? absint($_POST['post_id']) : '';
		$views_id = isset($_POST['scode_id']) ? absint($_POST['scode_id']) : '';
		$views_meta = get_post_meta($views_id, 'evs_venue_metaboxes', true);
		$popup_type = isset($views_meta['evs_popup_type']) ? $views_meta['evs_popup_type'] : 'single_popup';
		$popup_close_button = isset($views_meta['popup_close_button']) ? $views_meta['popup_close_button'] : '';
		$popup_venue_sorter = isset($views_meta['popup_venue_sorter']) ? $views_meta['popup_venue_sorter'] : '';
		$this_location_event = isset($popup_venue_sorter['this_location_event']) ? $popup_venue_sorter['this_location_event'] : '';
		$show_current_venue_events = isset($this_location_event['show_current_venue_events']) ? $this_location_event['show_current_venue_events'] : '';
		$par_page_event = isset($this_location_event['par_page_event']) ? $this_location_event['par_page_event'] : '';
		$show_lode_more_button = isset($this_location_event['show_lode_more_button']) ? $this_location_event['show_lode_more_button'] : '';
		$excerpt_length = isset($this_location_event['excerpt_length']) ? $this_location_event['excerpt_length'] : '';
		$lode_more_button_text = isset($this_location_event['lode_more_button_text']) ? $this_location_event['lode_more_button_text'] : '';

		// Image variables.
		$popup_evs_thumb = isset($popup_venue_sorter['popup_evs_thumb']) ? $popup_venue_sorter['popup_evs_thumb'] : '';
		$popup_evs_thumb_gray_scale = isset($popup_evs_thumb['popup_evs_thumb_gray_scale']) ? $popup_evs_thumb['popup_evs_thumb_gray_scale'] : '';
		// Title variables.
		$evs_title = isset($popup_venue_sorter['popup_evs_title']) ? $popup_venue_sorter['popup_evs_title'] : '';
		$event_count = isset($evs_title['popup_event_count']) ? $evs_title['popup_event_count'] : '';
		// Address variables.
		$evs_address = isset($popup_venue_sorter['popup_evs_address']) ? $popup_venue_sorter['popup_evs_address'] : '';
		$venue_address_icon = isset($evs_address['popup_venue_address_icon']) ? $evs_address['popup_venue_address_icon'] : '';
		$venue_address = isset($evs_address['popup_venue_address']) ? $evs_address['popup_venue_address'] : '';
		// Phone Number variables.
		$evs_phone = isset($popup_venue_sorter['popup_evs_phone']) ? $popup_venue_sorter['popup_evs_phone'] : '';
		$phone_number_icon = isset($evs_phone['popup_phone_number_icon']) ? $evs_phone['popup_phone_number_icon'] : '';
		// Website variables.
		$evs_website = isset($popup_venue_sorter['popup_evs_website']) ? $popup_venue_sorter['popup_evs_website'] : '';
		$website_icon = isset($evs_website['popup_website_icon']) ? $evs_website['popup_website_icon'] : '';
		
		$events_title = isset($this_location_event['show_events_title']) ? $this_location_event['show_events_title'] : '';
		$events_title_text = isset($this_location_event['events_title_text']) ? $this_location_event['events_title_text'] : '';

		$venue = get_post($post_id);
		$venue_id = $venue->ID;
		$paged = isset($_POST['page']) ? intval($_POST['page']) : 1;

		$args = array(
			'post_type' => 'tribe_events',
			'posts_per_page' => $par_page_event,
			'paged' => $paged,
			'meta_query' => array(
				array(
					'key' => '_EventVenueID',
					'value' => $venue_id,
					'compare' => '='
				)
			)
		);

		$events_query = new \WP_Query($args);
		$args['posts_per_page'] = -1;
		$events_count_query = new \WP_Query($args);
		$events_count = $events_count_query->post_count;



		wp_reset_postdata();
?>
		<div class="evs-modal evs-modal-<?php echo esc_attr($views_id); ?> white-popup">
			<div class="popup_ajax_content">


				<div class="evs-modal_wrapper" id="popup_ajax_content" data-event_count="<?php echo esc_attr($events_count) ?>" data-par_page="<?php echo esc_attr($par_page_event); ?>">
					<?php include Manager::evs_locate_template('venue/single-venue.php'); ?>
				</div>
			</div>
			<?php
			if ('multi_popup' === $popup_type) {
			?>
				<div class="evs-popup-button-prev"><i class="fa fa-chevron-left"></i></div>
				<div class="evs-popup-button-next"><i class="fa fa-chevron-right"></i></div>
			<?php
			}
			?>
		</div>
	<?php
		die();
	}
	/**
	 * Post next prev modal.
	 */
	public static function evs_post_admin_popup_next_prev()
	{
		if (isset($_POST['nonce']) && !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'])), 'evs_nonce')) {
			return false;
		}
		$post_id              	= isset($_POST['post_id']) ? absint($_POST['post_id']) : '';
		$views_id             	= isset($_POST['scode_id']) ? absint($_POST['scode_id']) : '';
		$views_meta         	= get_post_meta($views_id, 'evs_venue_metaboxes', true);
		$popup_close_button     = isset($views_meta['popup_close_button']) ? $views_meta['popup_close_button'] : '';
		$popup_venue_sorter     = isset($views_meta['popup_venue_sorter']) ? $views_meta['popup_venue_sorter'] : '';
		$this_location_event = isset($popup_venue_sorter['this_location_event']) ? $popup_venue_sorter['this_location_event'] : '';
		$show_current_venue_events = isset($this_location_event['show_current_venue_events']) ? $this_location_event['show_current_venue_events'] : '';
		$par_page_event = isset($this_location_event['par_page_event']) ? $this_location_event['par_page_event'] : '';
		$show_lode_more_button = isset($this_location_event['show_lode_more_button']) ? $this_location_event['show_lode_more_button'] : '';
		$excerpt_length = isset($this_location_event['excerpt_length']) ? $this_location_event['excerpt_length'] : '';
		$lode_more_button_text = isset($this_location_event['lode_more_button_text']) ? $this_location_event['lode_more_button_text'] : '';

		// Image variables.
		$popup_evs_thumb = isset($popup_venue_sorter['popup_evs_thumb']) ? $popup_venue_sorter['popup_evs_thumb'] : '';
		$popup_evs_thumb_gray_scale = isset($popup_evs_thumb['popup_evs_thumb_gray_scale']) ? $popup_evs_thumb['popup_evs_thumb_gray_scale'] : '';

		// Title variables.
		$evs_title = isset($popup_venue_sorter['popup_evs_title']) ? $popup_venue_sorter['popup_evs_title'] : '';
		$event_count = isset($evs_title['popup_event_count']) ? $evs_title['popup_event_count'] : '';

		// Address variables.
		$evs_address = isset($popup_venue_sorter['popup_evs_address']) ? $popup_venue_sorter['popup_evs_address'] : '';
		$venue_address_icon = isset($evs_address['popup_venue_address_icon']) ? $evs_address['popup_venue_address_icon'] : '';
		$venue_address = isset($evs_address['popup_venue_address']) ? $evs_address['popup_venue_address'] : '';
		// Phone variables.
		$evs_phone = isset($popup_venue_sorter['popup_evs_phone']) ? $popup_venue_sorter['popup_evs_phone'] : '';
		$phone_number_icon = isset($evs_phone['popup_phone_number_icon']) ? $evs_phone['popup_phone_number_icon'] : '';
		// website variables.
		$evs_website = isset($popup_venue_sorter['popup_evs_website']) ? $popup_venue_sorter['popup_evs_website'] : '';
		$website_icon = isset($evs_website['popup_website_icon']) ? $evs_website['popup_website_icon'] : '';

		$events_title = isset($this_location_event['show_events_title']) ? $this_location_event['show_events_title'] : '';
		$events_title_text = isset($this_location_event['events_title_text']) ? $this_location_event['events_title_text'] : '';

		$venue             = get_post($post_id);
		$venue_id = get_post($venue->ID);
		$paged = isset($_POST['page']) ? intval($_POST['page']) : 1;

		$args = array(
			'post_type' => 'tribe_events',
			'posts_per_page' => $par_page_event,
			'paged' => $paged,
			'meta_query' => array(
				array(
					'key' => '_EventVenueID',
					'value' => $venue->ID,
					'compare' => '='
				)
			)
		);

		$events_query = new \WP_Query($args);
		$args['posts_per_page'] = -1;
		$events_count_query = new \WP_Query($args);
		$events_count = $events_count_query->post_count;

		wp_reset_postdata();

	?>

		<div class="evs-modal_wrapper" id="popup_ajax_content" data-event_count="<?php echo esc_attr($events_count) ?>" data-par_page="<?php echo esc_attr($par_page_event); ?>">
			<?php include Manager::evs_locate_template('venue/single-venue.php'); ?>
		</div>
		<?php
		die();
	}

	public static function load_more_events()
	{
		if (isset($_POST['nonce']) && !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'])), 'evs_nonce')) {
			wp_send_json_error('Invalid nonce');
			die();
		}

		$post_id = isset($_POST['post_id']) ? absint($_POST['post_id']) : '';
		$paged = isset($_POST['page']) ? intval($_POST['page']) : 1;
		$views_id             	= isset($_POST['scode_id']) ? absint($_POST['scode_id']) : '';
		$views_meta = get_post_meta($views_id, 'evs_venue_metaboxes', true);
		$popup_venue_sorter     = isset($views_meta['popup_venue_sorter']) ? $views_meta['popup_venue_sorter'] : '';
		$this_location_event = isset($popup_venue_sorter['this_location_event']) ? $popup_venue_sorter['this_location_event'] : '';
		$par_page_event = isset($this_location_event['par_page_event']) ? $this_location_event['par_page_event'] : 3;
		$excerpt_length = isset($this_location_event['excerpt_length']) ? $this_location_event['excerpt_length'] : '';

		$evs_address = isset($popup_venue_sorter['popup_evs_address']) ? $popup_venue_sorter['popup_evs_address'] : '';
		$venue_google_mam_text = isset($evs_address['popup_venue_google_mam_text']) ? $evs_address['popup_venue_google_mam_text'] : '';

		$args = array(
			'post_type' => 'tribe_events',
			'posts_per_page' => $par_page_event,
			'paged' => $paged,
			'meta_query' => array(
				array(
					'key' => '_EventVenueID',
					'value' => $post_id,
					'compare' => '='
				)
			)
		);

		$events_query = new \WP_Query($args);

		if ($events_query->have_posts()) :
			while ($events_query->have_posts()) : $events_query->the_post();
				$event_date = tribe_get_start_date(null, false, 'M d');
				$date_parts = explode(' ', $event_date);
				$month = $date_parts[0];
				$day = $date_parts[1];
		?>
				<div class="list_wrapper" data-id="<?php echo esc_attr($post_id); ?>">
					<div class="event_date">
						<span class="ev-mo"><?php echo esc_html($month); ?></span>
						<span class="ev-day"><?php echo esc_html($day); ?></span>
					</div>
					<div class="event_details">
						<h3><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h3>
						<?php
						$excerpt = esc_html(get_the_excerpt());
						echo '<p>' . wp_kses_post(wp_trim_words($excerpt, $excerpt_length)) . '</p>';
						?>
					</div>
				</div>
<?php
			endwhile;
			wp_reset_postdata();
		else :
			wp_send_json_error('No more events');
		endif;

		die();
	}


	/**
	 * Generate and render shortcode.
	 *
	 * @param array $attributes Shortcode's all option.
	 * @since 1.0.0
	 */
	public function render_shortcode($attributes)
	{
		// Show nothing if the shortocode/view post is in trash or shortcode doesn't have any ID.
		if (empty($attributes['id']) || 'event_venue' !== get_post_type($attributes['id']) || ('trash' === get_post_status($attributes['id']))) {
			return;
		}
		$views_id           = esc_attr((int) $attributes['id']);
		$views_meta         = get_post_meta($views_id, 'evs_venue_metaboxes', true);
		$main_section_title = get_the_title($views_id);
		ob_start();
		$get_page_data      = self::get_page_data();
		$found_generator_id = $get_page_data['generator_id'];
		// This shortcode id not in page id option. Enqueue stylesheets in shortcode.

		if (!is_array($found_generator_id) || !$found_generator_id || !in_array($views_id, $found_generator_id)) {
			wp_enqueue_style('evs_venue_swiper');
			wp_enqueue_style('evs_venue_font_awesome');
			wp_enqueue_style('evs_venue_style');

			$dynamic_style = self::load_dynamic_style($views_id, $views_meta);
			echo '<style id="evs_venue_css' . esc_attr($views_id) . '">' . esc_html(wp_strip_all_tags($dynamic_style['dynamic_css'])) . '</style>';
		}

		self::db_options_update($views_id, $get_page_data);
		Manager::views_html($views_id, $views_meta, $main_section_title);
		return ob_get_clean();
	}
}
