<?php

/**
 * The file to manage all public-facing functionality of the plugin.
 *
 * @link       https://themeatelier.net
 *
 * @since      1.0.0
 *
 * @package event-venue-showcase
 * @subpackage event-venue-showcase/Frontend
 * @author     ThemeAtelier<themeatelierbd@gmail.com>
 */

namespace ThemeAtelier\EventVenueShowcase\Frontend;

use ThemeAtelier\EventVenueShowcase\Traits\ImageResize;

/**
 * The Manager class to manage all public facing stuffs.
 *
 * @since 1.0.0
 */
class Manager
{
    /**
     * Custom Template locator.
     *
     * @param  mixed $template_name template name.
     * @param  mixed $template_path template path.
     * @param  mixed $default_path default path.
     * @return string
     */
    public static function evs_locate_template($template_name, $template_path = '', $default_path = '')
    {
        if (!$template_path) {
            $template_path = 'event-venue-showcase/templates';
        }
        if (!$default_path) {
            $default_path = EVENT_VENUE_SHOWCASE_PATH . '/includes/Frontend/templates/';
        }
        $template = locate_template(
            array(
                trailingslashit($template_path) . $template_name,
                $template_name,
            )
        );
        if (!$template) {
            $template = $default_path . $template_name;
        }
        return $template;
    }

    /**
     * Minify output
     *
     * @param  string $html output minifier.
     * @return statement
     */
    public static function minify_output($html)
    {
        $html = preg_replace('/<!--(?!s*(?:[if [^]]+]|!|>))(?:(?!-->).)*-->/s', '', $html);
        $html = str_replace(array("\r\n", "\r", "\n", "\t"), '', $html);
        while (stristr($html, '  ')) {
            $html = str_replace('  ', ' ', $html);
        }
        return $html;
    }

    /**
     * View preloader
     *
     * @param  mixed $preloader Show/hide preloader.
     * @return void
     */
    public static function evs_venue_preloader($preloader)
    {
        ob_start();
        include self::evs_locate_template('preloader.php');
        $preloader = apply_filters('evs_venue_preloader', ob_get_clean());
        echo wp_kses_post($preloader);
    }

    /**
     * View Section Title.
     *
     * @param  integer $views_id The shortcode ID.
     * @param  integer $show_section_title The show / hide option.
     * @return void
     */
    public static function evs_venue_section_title($views_id, $show_section_title, $main_section_title)
    {
        $title_text = $main_section_title;
        $title      = apply_filters('evs_venue_section_title', $title_text);
        if (!empty($title) && $show_section_title) {
            include self::evs_locate_template('section-title.php');
        }
    }

    public static function evs_resize($url, $width = null, $height = null, $crop = null, $single = true, $upscale = false)
    {
        $evs_resize = ImageResize::getInstance();
        return $evs_resize->process($url, $width, $height, $crop, $single, $upscale);
    }

    /**
     * Venue Thumbnail.
     *
     * The function to process venue image HTML.
     *
     * @param object $venue The venue term object to show venue thumbnail.
     * @return void
     */
    public static function venue_logo($venue, $content_sorter, $views_meta)
    {
        $thumb_id = get_post_thumbnail_id($venue);

        if (!empty($thumb_id)) {
            $image_sizes       = $content_sorter['venue_image_size'];
            $post_image_width  = isset($content_sorter['evs_custom_image_size']['top']) ? $content_sorter['evs_custom_image_size']['top'] : '';
            $post_image_height = isset($content_sorter['evs_custom_image_size']['right']) ? $content_sorter['evs_custom_image_size']['right'] : '';
            $post_image_crop   = isset($content_sorter['evs_custom_image_size']['style']) ? $content_sorter['evs_custom_image_size']['style'] : '';

            $thumb_full_src    = wp_get_attachment_image_src($thumb_id, 'full');
            $thumb_full_src    = is_array($thumb_full_src) ? $thumb_full_src : array('', '', '');
            $image_src         = wp_get_attachment_image_src($thumb_id, $image_sizes);
            $image_src         = is_array($image_src) ? $image_src : array('', '', '');

            if (('custom' === $image_sizes) && (!empty($post_image_width) && $thumb_full_src[1] >= $post_image_width) && (!empty($post_image_height) && $thumb_full_src[2] >= $post_image_height)) {

                $hard_crop = 'Hard-crop' === $post_image_crop ? true : false;
                $image        = self::evs_resize($thumb_full_src[0], $post_image_width, $post_image_height, $hard_crop);
                $image_width  = $post_image_width;
                $image_height = $post_image_height;
            } else {
                $image        = !empty($image_src[0]) ? $image_src[0] : '';
                $image_width  = !empty($image_src[1]) ? $image_src[1] : 600;
                $image_height = !empty($image_src[2]) ? $image_src[2] : 400;
            }
        }

        ob_start();
        if (!empty($thumb_id)) {
            include self::evs_locate_template('venue/image.php');
        }
        $venue_logo = apply_filters('evs_venue_term_logo', ob_get_clean());
        echo wp_kses_post($venue_logo);
    }

    /**
     * Limit the the text.
     *
     * @param mixed  $text The text you want to limit.
     * @param int    $limit The number of words to display.
     * @param string $ellipsis The ellipsis at the end of the text.
     * @return statement
     */
    public static function evs_limit_text($text, $limit, $ellipsis = '...')
    {
        $word_arr = explode(' ', $text);
        if (count($word_arr) > $limit) {
            $words = implode(' ', array_slice($word_arr, 0, $limit)) . $ellipsis;
            return $words;
        } else {
            return $text;
        }
    }

    /**
     * Characters Limit of the text.
     *
     * @param mixed  $text The text you want to limit.
     * @param int    $limit The number of words to display.
     * @param string $ellipsis The ellipsis at the end of the text.
     * @return statement
     */
    public static function limit_content_chr($text, $limit, $ellipsis = '...')
    {
        $length = $limit;
        if (strlen($text) < $length + 10) {
            return $text; // don't cut if too short.
        }
        $visible = substr($text, 0, $length);
        return $visible . $ellipsis;
    }

    /**
     * The function to process views HTML.
     *
     * @param  integer $views_id The views generator ID.
     * @param  array   $views_meta The array of the generator serialized meta.
     * @return void
     */
    public static function views_html($views_id, $views_meta, $main_section_title)
    {
        $same_height = $views_meta['same_height'];
        $hide_empty_venue = $views_meta['hide_empty_venue'];

        include 'partials/configurations.php';

        $venue_args = array(
            'post_type' => 'tribe_venue',
            'posts_per_page' => $venue_limit,
            'orderby' => $order_by,
            'order' => $venue_order,
        );

        $venue_query = new \WP_Query($venue_args);
        $venue_query = $venue_query->posts;

        $venue_args['posts_per_page'] = -1;
        $venue_query_test = new \WP_Query($venue_args);
        $venue_query_test = $venue_query_test->posts;

        // Prepare an array to store venues without events
        $venues_without_events = array();

        // Loop through each venue and check if it has associated events
        foreach ($venue_query_test as $venue) {
            $event_args = array(
                'post_type' => 'tribe_events',
                'posts_per_page' => -1,
                'meta_query' => array(
                    array(
                        'key' => '_EventVenueID',
                        'value' => $venue->ID,
                        'compare' => '='
                    )
                )
            );

            $events_query = new \WP_Query($event_args);
            $events_count = $events_query->post_count;

            // If no events are found, add the venue to the array
            if ($events_count === 0) {
                $venues_without_events[] = $venue->ID;
            }
            wp_reset_postdata();
        }
        $venue_args['posts_per_page'] = $venue_limit;
        if ( ! empty( $evs_exclude_venue ) && $evs_filter_venue === 'exclude' ) {
            $evs_exclude_venue    = array_merge( $evs_exclude_venue, $venues_without_events );
            $venue_args['post__not_in'] = $evs_exclude_venue;
        } elseif ( ! empty( $evs_specific_venue ) && $evs_filter_venue === 'specific' ) {
            $evs_specific_venue   = array_diff( $evs_specific_venue, $venues_without_events );
            $venue_args['post__in'] = $evs_specific_venue;
        } else {
            $venue_args['post__not_in'] = $venues_without_events;
        }
        
        $venue_query = new \WP_Query($venue_args);
        $venue_query = $venue_query->posts;

        wp_reset_postdata();



        $carousel_class = 'carousel_layout' === $layout_preset ? ' evs-carousel-wrapper' : '';
?>
        <div class="evs-wrapper<?php echo esc_attr($carousel_class); ?>" id="evs_venue_main_area_<?php echo esc_attr($views_id); ?>" data-sid="<?php echo esc_attr($views_id); ?>">
            <div class="evs-section <?php echo $same_height ? 'evs_same_height' : ''; ?>">
                <?php
                self::evs_venue_preloader($preloader);
                self::evs_venue_section_title($views_id, $show_section_title, $main_section_title);
                switch ($layout_preset) {
                    case 'carousel_layout':
                        wp_enqueue_script('swiper-bundle');
                        wp_enqueue_script('magnific-popup');
                        wp_enqueue_script('evs-main');
                        include self::evs_locate_template('carousel.php');
                        break;
                }
                ?>
            </div>
        </div>
<?php
    } // views html.

}
